program define calculate_km_weights
/* edited by Marie in Dec 2020 to allow for more than 1 failure at event times (lines 32,34) 
   also added _d to line40 "gsort _t....", the case of no matching, to be consistent with line 45*/
   
	* args controls_per_case strata
    syntax [varlist(default=none)] , controls(integer)

version 13
preserve

// find all unique entry times
if ("`varlist'"=="") {
		keep _t0
} 
else {
		keep _t0 `varlist'
}

* _t0 is the entry time, here it will be renamed _t to enable later merging
qui duplicates drop 
rename _t0 _t
tempfile entrytimes
qui save `entrytimes'
* this file contains only the entry times (and matching factors, if any)

restore
* read back in the full data with standard stset names

// keep one row per event (or censoring) time
qui bys _t: keep if _n == 1


// Generate the probability not sampled at all of these times 
// (note that these will only be non-missing at EVENT times (i.e. riskset!=.))
qui gen prob_not_sampled = ln(1-(`controls'*numfail/(riskset - numfail)))
// Bug fix #32 - problem with ln(0) - add tiny increment
qui replace prob_not_sampled = -9e12 if `controls'*numfail/(riskset - numfail) == 1	


if ("`varlist'"=="") {
    // sort by event time and event status
	gsort _t _d
	// calculate cumulative log prob of not being sampled
	qui gen cumprob = sum(prob_not_sampled)
} 
else {
    bys `varlist' (_t _d): gen cumprob = sum(prob_not_sampled)
}

preserve
* contains the riskset and Log-prob of being sampled up to each event time
* this prob is a step function that will remain unchanged between events

// find entry time probs
if ("`varlist'"=="") {
		keep _t cumprob
} 
else {
		keep _t cumprob `varlist'
}

rename cumprob entryprob
qui duplicates drop 

// all entry times with entry probs
if ("`varlist'"=="") {
		qui merge 1:1 _t using `entrytimes'
} 
* this should result in all unmerged records 
* (i.e. either an entry time _t from "entrytimes"
*  or an event/censoring time _t in the working data set
* as these are unlikely to ever be the same time
else {
		qui merge 1:1 _t `varlist' using `entrytimes' 
}

drop _merge

* all event/censoring times will have a cumprob as a result of the previous step,
* but entry times will not.
* 
// allocate closest previous prob to each entry time
if ("`varlist'"=="") {
	sort _t
	qui replace entryprob = entryprob[_n -1] if entryprob == .

} 
else {
	bys `varlist' (_t): replace entryprob = entryprob[_n -1] if entryprob == .
}

qui replace entryprob = 0 if entryprob == .
* return the correct name to the entry time (_t0)
rename _t _t0
tempfile entryprob
qui save `entryprob'

restore
* remember this data contains the riskset and Log-prob of being sampled 
* up to each event/censoring time (a step function at events)
* so by merging with the probabilities of being selected up to the entry time
* we can find the probability of being selected between entry and exit



if ("`varlist'"=="") {
	qui merge m:1 _t0 using `entryprob'
} 
else {
	qui merge m:1 _t0 `varlist' using `entryprob'
}

qui keep if _t < .
drop _merge

// exponentiate probabilities and calculate weights
qui gen finalprob = 1 - exp(cumprob - entryprob)
qui gen weights = 1 / finalprob
qui replace weights = 1 if _d == 1
sort _t
drop cumprob entryprob finalprob prob_not_sampled

end
