program define cohort_km_weights
/* changes in Dec 2020 by Marie to:
 create a variable with the number of failures at each event time (lines 23, 27)
  save this variable in the created data set (lines 62, 65) 
  keep the stratification variables in the ID and time data used for final merge (line 37)
  use "qui" for final merge command (lines 68 and 71)
  */
	* args controls_per_case strata
    syntax [varlist(default=none)] , controls(integer) id(string) [entry(string)] time(string) failure(string)

version 13

// save original data in case of error
tempfile inputdata
qui save `inputdata'

// stset data
qui stset `time', failure(`failure') entry(`entry')

// calculate the risksets
if ("`varlist'"=="") {
    sts generate riskset = n
    sts generate numfail = d 
} 
else {
    sts generate riskset = n, by("`varlist'")
    sts generate numfail = d, by("`varlist'")
}

// save the ID and time info in this full data for later
preserve

if ("`varlist'"=="") {
keep `id' `entry' `time'
}
else {
  keep `varlist' `id' `entry' `time'
}

tempfile idtime
save `idtime'


restore
capture {
    calculate_km_weights `varlist', controls(`controls')
}


// capture error from calculate_km_weights
  local rc = _rc
	if `rc' != 0 {
	  qui use `inputdata', clear
	  error `rc'
	}

// prepare unique output rows
// include unique combinations of entry/exit times, event status
// and matching variables where applicable

if ("`varlist'"=="") {
  keep `entry'  `time' `failure' numfail riskset weights
}
else {
  keep `varlist'  `entry' `time' `failure' numfail riskset weights
}
if ("`varlist'"=="") {
  qui merge 1:m  `entry'  `time' using `idtime'
}
else {
  qui merge 1:m `varlist'  `entry' `time' using `idtime' 
}
 
 drop _merge
 
 sort `id'
 
 order `id' `varlist'  `entry' `time' `failure' numfail riskset weights
 
 discard
  
end
