program define sampled_km_weights
/* modified by Marie Dec 2020 to accommodate:
     matching strata in the merge to risk sets (line14)
     shared failure times in the NCC data (line 28) */
	* args controls_per_case strata
    syntax [varlist(default=none)] using/ , controls(integer) id(string) [entry(string)] time(string) failure(string) riskset(string) numfail(string)

version 13	

// save original data in case of error
tempfile inputdata
qui save `inputdata'


// merge sampled cases and controls to the external riskset information
	qui merge m:1 `time' `varlist' using `using'
	/* note this merge needs to accommodate matching*/
	
	// remove rows which are only present in the riskset data
	qui drop if _merge == 2
	qui drop _merge

// stset data
qui stset `time', failure(`failure') entry(`entry')

// rename riskset variable and numfail variable
qui rename `riskset' riskset
qui rename `numfail' numfail
preserve

capture {
  calculate_km_weights `varlist', controls(`controls')
}

// capture error from calculate_km_weights
  local rc = _rc
	if `rc' != 0 {
	  qui use `inputdata', clear
	  error `rc'
	}
	
tempfile nccweights
qui save `nccweights'

restore

// merge weights back to original input data
qui merge m:1 _t _d _t0 using `nccweights'


// drop rows that are only present in the using data
qui drop if _merge == 2

// check for duplicate entries for subjects
qui duplicates report `id'

if r(unique_value) != r(N) {
	di as error "Warning: Subjects are represented more than once in the returned data set"
}

drop _st _d _t _t0 _merge

discard

end
